@@ -9,7 +9,7 @@
  **  This source code is part of the Valentina project, a pattern making
  **  program, whose allow create and modeling patterns of clothing.
  **  Copyright (C) 2015 Valentina project
- **  <https://bitbucket.org/dismine/valentina> All Rights Reserved.
+ **  <https://gitlab.com/smart-pattern/valentina> All Rights Reserved.
  **
  **  Valentina is free software: you can redistribute it and/or modify
  **  it under the terms of the GNU General Public License as published by
@@ -34,11 +34,11 @@
 #include "../ifc/exception/vexceptionconversionerror.h"
 #include "../ifc/exception/vexceptionemptyparameter.h"
 #include "../ifc/exception/vexceptionwrongid.h"
-#include "../vmisc/logging.h"
 #include "../vmisc/vsysexits.h"
 #include "../vmisc/diagnostic.h"
 #include "../vmisc/qt_dispatch/qt_dispatch.h"
 #include "../qmuparser/qmuparsererror.h"
+#include "../vpatterndb/variables/vmeasurement.h"
 
 #include <QDir>
 #include <QFileOpenEvent>
@@ -53,6 +53,10 @@
 #include <QSpacerItem>
 #include <QThread>
 
+#if defined(APPIMAGE) && defined(Q_OS_LINUX)
+#   include "../vmisc/appimage.h"
+#endif // defined(APPIMAGE) && defined(Q_OS_LINUX)
+
 QT_WARNING_PUSH
 QT_WARNING_DISABLE_CLANG("-Wmissing-prototypes")
 QT_WARNING_DISABLE_INTEL(1418)
@@ -146,23 +150,30 @@ inline void noisyFailureMsgHandler(QtMsgType type, const QMessageLogContext &con
         type = QtDebugMsg;
     }
 
+    QString logMsg = msg;
+    const bool isWarningMessage = VAbstractApplication::VApp()->IsWarningMessage(msg);
+    if (isWarningMessage)
+    {
+        logMsg = logMsg.remove(VAbstractApplication::warningMessageSignature);
+    }
+
     switch (type)
     {
         case QtDebugMsg:
-            vStdOut() << QApplication::translate("mNoisyHandler", "DEBUG:") << msg << "\n";
+            vStdOut() << QApplication::translate("mNoisyHandler", "DEBUG:") << logMsg << "\n";
             return;
         case QtWarningMsg:
-            vStdErr() << QApplication::translate("mNoisyHandler", "WARNING:") << msg << "\n";
+            vStdErr() << QApplication::translate("mNoisyHandler", "WARNING:") << logMsg << "\n";
             break;
         case QtCriticalMsg:
-            vStdErr() << QApplication::translate("mNoisyHandler", "CRITICAL:") << msg << "\n";
+            vStdErr() << QApplication::translate("mNoisyHandler", "CRITICAL:") << logMsg << "\n";
             break;
         case QtFatalMsg:
-            vStdErr() << QApplication::translate("mNoisyHandler", "FATAL:") << msg << "\n";
+            vStdErr() << QApplication::translate("mNoisyHandler", "FATAL:") << logMsg << "\n";
             break;
-        #if QT_VERSION > QT_VERSION_CHECK(5, 4, 2)
+        #if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
         case QtInfoMsg:
-            vStdOut() << QApplication::translate("mNoisyHandler", "INFO:") << msg << "\n";
+            vStdOut() << QApplication::translate("mNoisyHandler", "INFO:") << logMsg << "\n";
             break;
         #endif
         default:
@@ -193,7 +204,7 @@ inline void noisyFailureMsgHandler(QtMsgType type, const QMessageLogContext &con
                 messageBox.setWindowTitle(QApplication::translate("mNoisyHandler", "Fatal error"));
                 messageBox.setIcon(QMessageBox::Critical);
                 break;
-            #if QT_VERSION > QT_VERSION_CHECK(5, 4, 2)
+            #if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
             case QtInfoMsg:
                 messageBox.setWindowTitle(QApplication::translate("mNoisyHandler", "Information"));
                 messageBox.setIcon(QMessageBox::Information);
@@ -208,11 +219,11 @@ inline void noisyFailureMsgHandler(QtMsgType type, const QMessageLogContext &con
 
         if (type == QtWarningMsg || type == QtCriticalMsg || type == QtFatalMsg)
         {
-            if (not qApp->IsTestMode())
+            if (not MApplication::VApp()->IsTestMode())
             {
                 if (topWinAllowsPop)
                 {
-                    messageBox.setText(VAbstractApplication::ClearMessage(msg));
+                    messageBox.setText(VAbstractApplication::ClearMessage(logMsg));
                     messageBox.setStandardButtons(QMessageBox::Ok);
                     messageBox.setWindowModality(Qt::ApplicationModal);
                     messageBox.setModal(true);
@@ -331,6 +342,11 @@ bool MApplication::notify(QObject *receiver, QEvent *event)
                    qUtf8Printable(e.ErrorMessage()), qUtf8Printable(e.DetailedInformation()));
         return true;
     }
+    catch (const qmu::QmuParserWarning &e)
+    {
+        qCCritical(mApp, "%s", qUtf8Printable(tr("Formula warning: %1. Program will be terminated.").arg(e.GetMsg())));
+        exit(V_EX_DATAERR);
+    }
     // These last two cases special. I found that we can't show here modal dialog with error message.
     // Somehow program doesn't waite untile an error dialog will be closed. But if ignore this program will hang.
     catch (const qmu::QmuParserError &e)
@@ -409,9 +425,30 @@ void MApplication::InitOptions()
        //This does not happen under GNOME or KDE
        QIcon::setThemeName("win.icon.theme");
     }
-
+    ActivateDarkMode();
     QResource::registerResource(diagramsPath());
 }
+// Dark mode
+
+void MApplication::ActivateDarkMode()
+{
+    VTapeSettings *settings = MApplication::VApp()->TapeSettings();
+     if (settings->GetDarkMode())
+     {
+         QFile f(":qdarkstyle/style.qss");
+         if (!f.exists())
+         {
+             qDebug()<<"Unable to set stylesheet, file not found\n";
+         }
+         else
+         {
+             f.open(QFile::ReadOnly | QFile::Text);
+             QTextStream ts(&f);
+             qApp->setStyleSheet(ts.readAll());
+         }
+
+     }
+}
 
 //---------------------------------------------------------------------------------------------------------------------
 void MApplication::InitTrVars()
@@ -466,6 +503,15 @@ bool MApplication::event(QEvent *e)
     return VAbstractApplication::event(e);
 }
 
+//---------------------------------------------------------------------------------------------------------------------
+void MApplication::AboutToQuit()
+{
+    // If try to use the method QApplication::exit program can't sync settings and show warning about QApplication
+    // instance. Solution is to call sync() before quit.
+    // Connect this slot with VApplication::aboutToQuit.
+    Settings()->sync();
+}
+
 //---------------------------------------------------------------------------------------------------------------------
 void MApplication::OpenSettings()
 {
@@ -501,7 +547,7 @@ QString MApplication::diagramsPath() const
         }
         else
         {
-            return QStringLiteral("/usr/share/valentina") + dPath;
+            return PKGDATADIR + dPath;
         }
     }
 #else // Unix
@@ -512,7 +558,12 @@ QString MApplication::diagramsPath() const
     }
     else
     {
-        return QStringLiteral("/usr/share/valentina") + dPath;
+#if defined(APPIMAGE) && defined(Q_OS_LINUX)
+        /* Fix path to diagrams when run inside AppImage. */
+        return AppImageRoot() + PKGDATADIR + dPath;
+#else
+        return PKGDATADIR + dPath;
+#endif // defined(APPIMAGE) && defined(Q_OS_LINUX)
     }
 #endif
 }
@@ -560,96 +611,46 @@ void MApplication::ParseCommandLine(const SocketConnection &connection, const QS
     parser.addHelpOption();
     parser.addVersionOption();
     parser.addPositionalArgument("filename", tr("The measurement file."));
-    //-----
-    QCommandLineOption heightOption(QStringList() << "e" << "height",
-            tr("Open with the base height. Valid values: %1cm.")
-                                    .arg(VMeasurement::WholeListHeights(Unit::Cm).join(", ")),
-            tr("The base height"));
-    parser.addOption(heightOption);
-    //-----
-    QCommandLineOption sizeOption(QStringList() << "s" << "size",
-            tr("Open with the base size. Valid values: %1cm.").arg(VMeasurement::WholeListSizes(Unit::Cm).join(", ")),
-            tr("The base size"));
-    parser.addOption(sizeOption);
-    //-----
-    QCommandLineOption unitOption(QStringList() << "u" << "unit",
-            tr("Set pattern file unit: cm, mm, inch."),
-            tr("The pattern unit"));
-    parser.addOption(unitOption);
-    //-----
-    QCommandLineOption testOption(QStringList() << "test",
-            tr("Use for unit testing. Run the program and open a file without showing the main window."));
-    parser.addOption(testOption);
-    //-----
-    QCommandLineOption scalingOption(QStringList() << LONG_OPTION_NO_HDPI_SCALING,
-            tr("Disable high dpi scaling. Call this option if has problem with scaling (by default scaling enabled). "
-               "Alternatively you can use the %1 environment variable.").arg("QT_AUTO_SCREEN_SCALE_FACTOR=0"));
-    parser.addOption(scalingOption);
-    //-----
+
+    const QString LONG_OPTION_DIMENSION_A = QStringLiteral("dimensionA");
+    const QString SINGLE_OPTION_DIMENSION_A = QChar('a');
+
+    const QString LONG_OPTION_DIMENSION_B = QStringLiteral("dimensionB");
+    const QString SINGLE_OPTION_DIMENSION_B = QChar('b');
+
+    const QString LONG_OPTION_DIMENSION_C = QStringLiteral("dimensionC");
+    const QString SINGLE_OPTION_DIMENSION_C = QChar('c');
+
+    const QString LONG_OPTION_UNITS = QStringLiteral("units");
+    const QString SINGLE_OPTION_UNITS = QChar('u');
+
+    const QString LONG_OPTION_TEST = QStringLiteral("test");
+
+    parser.addOptions(
+    {
+        {{SINGLE_OPTION_DIMENSION_A, LONG_OPTION_DIMENSION_A}, tr("Set base for dimension A in the table units."),
+             tr("The dimension A base")},
+
+        {{SINGLE_OPTION_DIMENSION_B, LONG_OPTION_DIMENSION_B}, tr("Set base for dimension B in the table units."),
+             tr("The dimension B base")},
+
+        {{SINGLE_OPTION_DIMENSION_C, LONG_OPTION_DIMENSION_C}, tr("Set base for dimension C in the table units."),
+             tr("The dimension C base")},
+
+        {{SINGLE_OPTION_UNITS, LONG_OPTION_UNITS}, tr("Set pattern file units: cm, mm, inch."),
+             tr("The pattern units")},
+
+        {LONG_OPTION_TEST,
+             tr("Use for unit testing. Run the program and open a file without showing the main window.")},
+
+        {LONG_OPTION_NO_HDPI_SCALING,
+             tr("Disable high dpi scaling. Call this option if has problem with scaling (by default scaling enabled). "
+                "Alternatively you can use the %1 environment variable.").arg("QT_AUTO_SCREEN_SCALE_FACTOR=0")},
+    });
+
     parser.process(arguments);
 
-    bool flagHeight = false;
-    bool flagSize = false;
-    bool flagUnit = false;
-
-    int size = 0;
-    int height = 0;
-    Unit unit = Unit::Cm;
-
-    if (parser.isSet(heightOption))
-    {
-        const QString heightValue = parser.value(heightOption);
-        if (VMeasurement::IsGradationHeightValid(heightValue))
-        {
-            flagHeight = true;
-            height = heightValue.toInt();
-        }
-        else
-        {
-            qCCritical(mApp, "%s\n",
-                    qPrintable(tr("Invalid base height argument. Must be %1cm.")
-                               .arg(VMeasurement::WholeListHeights(Unit::Cm).join(", "))));
-            parser.showHelp(V_EX_USAGE);
-        }
-    }
-
-    if (parser.isSet(sizeOption))
-    {
-        const QString sizeValue = parser.value(sizeOption);
-        if (VMeasurement::IsGradationSizeValid(sizeValue))
-        {
-            flagSize = true;
-            size = sizeValue.toInt();
-        }
-        else
-        {
-            qCCritical(mApp, "%s\n",
-                    qPrintable(tr("Invalid base size argument. Must be %1cm.")
-                               .arg(VMeasurement::WholeListSizes(Unit::Cm).join(", "))));
-            parser.showHelp(V_EX_USAGE);
-        }
-    }
-
-    {
-    const QString unitValue = parser.value(unitOption);
-    if (not unitValue.isEmpty())
-    {
-
-        const QStringList units = QStringList() << unitMM << unitCM << unitINCH;
-        if (units.contains(unitValue))
-        {
-            flagUnit = true;
-            unit = StrToUnits(unitValue);
-        }
-        else
-        {
-            qCCritical(mApp, "%s\n", qPrintable(tr("Invalid base size argument. Must be cm, mm or inch.")));
-            parser.showHelp(V_EX_USAGE);
-        }
-    }
-    }
-
-    testMode = parser.isSet(testOption);
+    testMode = parser.isSet(LONG_OPTION_TEST);
 
     if (not testMode && connection == SocketConnection::Client)
     {
@@ -698,6 +699,84 @@ void MApplication::ParseCommandLine(const SocketConnection &connection, const QS
             parser.showHelp(V_EX_USAGE);
         }
 
+        bool flagDimensionA = false;
+        bool flagDimensionB = false;
+        bool flagDimensionC = false;
+        bool flagUnits = false;
+
+        int dimensionAValue = 0;
+        int dimensionBValue = 0;
+        int dimensionCValue = 0;
+        Unit unit = Unit::Cm;
+
+        if (parser.isSet(LONG_OPTION_DIMENSION_A))
+        {
+            const QString value = parser.value(LONG_OPTION_DIMENSION_A);
+
+            bool ok = false;
+            dimensionAValue = value.toInt(&ok);
+            if(ok && dimensionAValue > 0)
+            {
+                flagDimensionA = true;
+            }
+            else
+            {
+                qCCritical(mApp, "%s\n", qPrintable(tr("Invalid dimension A base value.")));
+                parser.showHelp(V_EX_USAGE);
+            }
+        }
+
+        if (parser.isSet(LONG_OPTION_DIMENSION_B))
+        {
+            const QString value = parser.value(LONG_OPTION_DIMENSION_B);
+
+            bool ok = false;
+            dimensionBValue = value.toInt(&ok);
+            if(ok && dimensionBValue > 0)
+            {
+                flagDimensionB = true;
+            }
+            else
+            {
+                qCCritical(mApp, "%s\n", qPrintable(tr("Invalid dimension B base value.")));
+                parser.showHelp(V_EX_USAGE);
+            }
+        }
+
+        if (parser.isSet(LONG_OPTION_DIMENSION_C))
+        {
+            const QString value = parser.value(LONG_OPTION_DIMENSION_C);
+
+            bool ok = false;
+            dimensionCValue = value.toInt(&ok);
+            if(ok && dimensionCValue > 0)
+            {
+                flagDimensionC = true;
+            }
+            else
+            {
+                qCCritical(mApp, "%s\n", qPrintable(tr("Invalid dimension C base value.")));
+                parser.showHelp(V_EX_USAGE);
+            }
+        }
+
+        {
+            const QString unitValue = parser.value(LONG_OPTION_UNITS);
+            if (not unitValue.isEmpty())
+            {
+                if (QStringList{unitMM, unitCM, unitINCH}.contains(unitValue))
+                {
+                    flagUnits = true;
+                    unit = StrToUnits(unitValue);
+                }
+                else
+                {
+                    qCCritical(mApp, "%s\n", qPrintable(tr("Invalid base size argument. Must be cm, mm or inch.")));
+                    parser.showHelp(V_EX_USAGE);
+                }
+            }
+        }
+
         for (auto &arg : args)
         {
             NewMainWindow();
@@ -711,17 +790,31 @@ void MApplication::ParseCommandLine(const SocketConnection &connection, const QS
                 continue;
             }
 
-            if (flagSize)
+            if (flagDimensionA)
             {
-                MainWindow()->SetBaseMSize(size);
+                if (not MainWindow()->SetDimensionABase(dimensionAValue))
+                {
+                    parser.showHelp(V_EX_USAGE);
+                }
             }
 
-            if (flagHeight)
+            if (flagDimensionB)
             {
-                MainWindow()->SetBaseMHeight(height);
+                if (not MainWindow()->SetDimensionBBase(dimensionBValue))
+                {
+                    parser.showHelp(V_EX_USAGE);
+                }
             }
 
-            if (flagUnit)
+            if (flagDimensionC)
+            {
+                if (not MainWindow()->SetDimensionCBase(dimensionCValue))
+                {
+                    parser.showHelp(V_EX_USAGE);
+                }
+            }
+
+            if (flagUnits)
             {
                 MainWindow()->SetPUnit(unit);
             }
@@ -746,12 +839,18 @@ void MApplication::ParseCommandLine(const SocketConnection &connection, const QS
     }
 }
 
+//---------------------------------------------------------------------------------------------------------------------
+auto MApplication::VApp() -> MApplication *
+{
+    return qobject_cast<MApplication*>(QCoreApplication::instance());
+}
+
 //---------------------------------------------------------------------------------------------------------------------
 TMainWindow *MApplication::NewMainWindow()
 {
     TMainWindow *tape = new TMainWindow();
     mainWindows.prepend(tape);
-    if (not qApp->IsTestMode())
+    if (not MApplication::VApp()->IsTestMode())
     {
         tape->show();
     }
