@@ -98,6 +98,7 @@ public:
 
     virtual QString GenerateLabel(const LabelType &type, const QString &reservedName = QString())const override;
     virtual QString GenerateSuffix() const override;
+    virtual void    Clear() override;
 
     bool IsReadOnly() const;
     void SetReadOnly(bool rOnly);
@@ -118,12 +119,11 @@ signals:
 public slots:
     virtual void LiteParseTree(const Document &parse) override;
 
+    void RefreshPieceGeometry();
+
 protected:
     virtual void   customEvent(QEvent * event) override;
 
-private slots:
-    void RefreshPieceGeometry();
-
 private:
     Q_DISABLE_COPY(VPattern)
 
