@@ -138,6 +138,10 @@ const QString pl_size          = QStringLiteral("size");
 const QString pl_height        = QStringLiteral("height");
 const QString pl_hip           = QStringLiteral("hip");
 const QString pl_waist         = QStringLiteral("waist");
+const QString pl_sizeLabel     = QStringLiteral("sizeLabel");
+const QString pl_heightLabel   = QStringLiteral("heightLabel");
+const QString pl_hipLabel      = QStringLiteral("hipLabel");
+const QString pl_waistLabel    = QStringLiteral("waistLabel");
 const QString pl_date          = QStringLiteral("date");
 const QString pl_time          = QStringLiteral("time");
 const QString pl_birthDate     = QStringLiteral("birthDate");
@@ -623,7 +627,8 @@ void InitLanguages(QComboBox *combobox)
 
         QLocale loc = QLocale(locale);
         QString lang = loc.nativeLanguageName();
-        QIcon ico(QString("%1/%2.png").arg("://flags", QLocale::countryToString(loc.country())));
+        // Since Qt 5.12 country names have spaces
+        QIcon ico(QString("://flags/%1.png").arg(QLocale::countryToString(loc.country()).remove(' ')));
 
         combobox->addItem(ico, lang, locale);
     }
@@ -631,7 +636,8 @@ void InitLanguages(QComboBox *combobox)
     if (combobox->count() == 0 || not englishUS)
     {
         // English language is internal and doens't have own *.qm file.
-        QIcon ico(QString("%1/%2.png").arg("://flags", QLocale::countryToString(QLocale::UnitedStates)));
+        // Since Qt 5.12 country names have spaces
+        QIcon ico(QString("://flags/%1.png").arg(QLocale::countryToString(QLocale::UnitedStates).remove(' ')));
         QString lang = QLocale(en_US).nativeLanguageName();
         combobox->addItem(ico, lang, en_US);
     }
