@@ -406,6 +406,10 @@ extern const QString pl_size;
 extern const QString pl_height;
 extern const QString pl_hip;
 extern const QString pl_waist;
+extern const QString pl_sizeLabel;
+extern const QString pl_heightLabel;
+extern const QString pl_hipLabel;
+extern const QString pl_waistLabel;
 extern const QString pl_date;
 extern const QString pl_time;
 extern const QString pl_birthDate;
