@@ -38,6 +38,7 @@
 #include "../vmisc/compatibility.h"
 #include "../ifc/exception/vexceptioninvalidnotch.h"
 #include "../vlayout/testpath.h"
+#include "../ifc/xml/vabstractpattern.h"
 
 #include <QSharedPointer>
 #include <QDebug>
@@ -1106,6 +1107,7 @@ VPassmark VPiece::CreatePassmark(const QVector<VPieceNode> &path, int previousIn
     passmarkData.isShowSecondPassmark = path.at(passmarkIndex).IsShowSecondPassmark();
     passmarkData.passmarkIndex = passmarkIndex;
     passmarkData.id = path.at(passmarkIndex).GetId();
+    passmarkData.globalPassmarkLength = ToPixel(GlobalPassmarkLength(data), *data->GetPatternUnit());
 
     return VPassmark(passmarkData);
 }
@@ -1170,6 +1172,39 @@ QJsonObject VPiece::DBToJson(const VContainer *data) const
     return dbObject;
 }
 
+//---------------------------------------------------------------------------------------------------------------------
+auto VPiece::GlobalPassmarkLength(const VContainer *data) const -> qreal
+{
+    qreal length = 0;
+    QString passmarkLengthVariable = VAbstractValApplication::VApp()->getCurrentDocument()->GetPassmarkLengthVariable();
+    if (passmarkLengthVariable.isEmpty())
+    {
+        return 0;
+    }
+
+    try
+    {
+        QSharedPointer<VInternalVariable> var = data->GetVariable<VInternalVariable>(passmarkLengthVariable);
+        length = *var->GetValue();
+
+        if (length <= accuracyPointOnLine)
+        {
+            const QString errorMsg = QObject::tr("Invalid global value for a passmark length. Piece '%1'. Length is "
+                                                 "less than minimal allowed.")
+                    .arg(GetName());
+            VAbstractApplication::VApp()->IsPedantic()
+                    ? throw VException(errorMsg)
+                    : qWarning() << VAbstractValApplication::warningMessageSignature + errorMsg;
+        }
+    }
+    catch (const VExceptionBadId &)
+    {
+        length = 0;
+    }
+
+    return length;
+}
+
 //---------------------------------------------------------------------------------------------------------------------
 void VPiece::DumpPiece(const VPiece &piece, const VContainer *data)
 {
@@ -1178,6 +1213,16 @@ void VPiece::DumpPiece(const VPiece &piece, const VContainer *data)
     temp.setAutoRemove(false); // Remove dump manually
     if (temp.open())
     {
+#if defined(Q_OS_LINUX)
+    #if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+//        On Linux, QTemporaryFile will attempt to create unnamed temporary
+//        files. If that succeeds, open() will return true but exists() will be
+//        false. If you call fileName() or any function that calls it,
+//        QTemporaryFile will give the file a name, so most applications will
+//        not see a difference.
+        temp.fileName(); // call to create a file on disk
+    #endif
+#endif
         QJsonObject testCase
         {
             {"bd", piece.DBToJson(data)},
