@@ -28,7 +28,7 @@
 
 #include "dialogarc.h"
 
-#include <limits.h>
+#include <climits>
 #include <QDialog>
 #include <QLabel>
 #include <QPlainTextEdit>
