@@ -28,7 +28,7 @@
 
 #include "dialogpointofintersectioncircles.h"
 
-#include <limits.h>
+#include <climits>
 #include <QColor>
 #include <QComboBox>
 #include <QDialog>
