@@ -28,7 +28,7 @@
 
 #include "vtoolcurveintersectaxis.h"
 
-#include <limits.h>
+#include <climits>
 #include <QLineF>
 #include <QMap>
 #include <QRectF>
@@ -76,7 +76,10 @@ VToolCurveIntersectAxis::VToolCurveIntersectAxis(const VToolCurveIntersectAxisIn
     :VToolLinePoint(initData.doc, initData.data, initData.id, initData.typeLine, initData.lineColor, QString(),
                     initData.basePointId, 0, initData.notes, parent),
       formulaAngle(initData.formulaAngle),
-      curveId(initData.curveId)
+      curveId(initData.curveId),
+      m_segments(initData.segments),
+      m_aliasSuffix1(initData.aliasSuffix1),
+      m_aliasSuffix2(initData.aliasSuffix2)
 {
     ToolCreation(initData.typeCreation);
 }
@@ -96,6 +99,8 @@ void VToolCurveIntersectAxis::setDialog()
     dialogTool->setCurveId(curveId);
     dialogTool->SetPointName(p->name());
     dialogTool->SetNotes(m_notes);
+    dialogTool->SetAliasSuffix1(m_aliasSuffix1);
+    dialogTool->SetAliasSuffix2(m_aliasSuffix2);
 }
 
 //---------------------------------------------------------------------------------------------------------------------
@@ -120,6 +125,8 @@ VToolCurveIntersectAxis *VToolCurveIntersectAxis::Create(const QPointer<DialogTo
     initData.parse = Document::FullParse;
     initData.typeCreation = Source::FromGui;
     initData.notes = dialogTool->GetNotes();
+    initData.aliasSuffix1 = dialogTool->GetAliasSuffix1();
+    initData.aliasSuffix2 = dialogTool->GetAliasSuffix2();
 
     VToolCurveIntersectAxis *point = Create(initData);
     if (point != nullptr)
@@ -150,7 +157,7 @@ VToolCurveIntersectAxis *VToolCurveIntersectAxis::Create(VToolCurveIntersectAxis
 
     const qreal segLength = curve->GetLengthByPoint(fPoint);
 
-    VPointF *p = new VPointF(fPoint, initData.name, initData.mx, initData.my);
+    auto *p = new VPointF(fPoint, initData.name, initData.mx, initData.my);
     p->SetShowLabel(initData.showLabel);
 
     if (initData.typeCreation == Source::FromGui)
@@ -160,14 +167,16 @@ VToolCurveIntersectAxis *VToolCurveIntersectAxis::Create(VToolCurveIntersectAxis
 
         initData.data->getNextId();
         initData.data->getNextId();
-        InitSegments(curve->getType(), segLength, p, initData.curveId, initData.data);
+        initData.segments = VToolSinglePoint::InitSegments(curve->getType(), segLength, p, initData.curveId,
+                                                           initData.data, initData.aliasSuffix1, initData.aliasSuffix2);
     }
     else
     {
         initData.data->UpdateGObject(initData.id, p);
         initData.data->AddLine(initData.basePointId, initData.id);
 
-        InitSegments(curve->getType(), segLength, p, initData.curveId, initData.data);
+        initData.segments = VToolSinglePoint::InitSegments(curve->getType(), segLength, p, initData.curveId,
+                                                           initData.data, initData.aliasSuffix1, initData.aliasSuffix2);
 
         if (initData.parse != Document::FullParse)
         {
@@ -178,7 +187,7 @@ VToolCurveIntersectAxis *VToolCurveIntersectAxis::Create(VToolCurveIntersectAxis
     if (initData.parse == Document::FullParse)
     {
         VAbstractTool::AddRecord(initData.id, Tool::CurveIntersectAxis, initData.doc);
-        VToolCurveIntersectAxis *point = new VToolCurveIntersectAxis(initData);
+        auto *point = new VToolCurveIntersectAxis(initData);
         initData.scene->addItem(point);
         InitToolConnections(initData.scene, point);
         VAbstractPattern::AddTool(initData.id, point);
@@ -186,6 +195,10 @@ VToolCurveIntersectAxis *VToolCurveIntersectAxis::Create(VToolCurveIntersectAxis
         initData.doc->IncrementReferens(curve->getIdTool());
         return point;
     }
+
+    auto *tool = qobject_cast<VToolCurveIntersectAxis *>(VAbstractPattern::getTool(initData.id));
+    tool->SetSegments(initData.segments);
+
     return nullptr;
 }
 
@@ -264,6 +277,10 @@ void VToolCurveIntersectAxis::SaveDialog(QDomElement &domElement, QList<quint32>
     doc->SetAttribute(domElement, AttrAngle, dialogTool->GetAngle());
     doc->SetAttribute(domElement, AttrBasePoint, QString().setNum(dialogTool->GetBasePointId()));
     doc->SetAttribute(domElement, AttrCurve, QString().setNum(dialogTool->getCurveId()));
+    doc->SetAttributeOrRemoveIf(domElement, AttrAlias1, dialogTool->GetAliasSuffix1(),
+                                dialogTool->GetAliasSuffix1().isEmpty());
+    doc->SetAttributeOrRemoveIf(domElement, AttrAlias2, dialogTool->GetAliasSuffix2(),
+                                dialogTool->GetAliasSuffix2().isEmpty());
 
     const QString notes = dialogTool->GetNotes();
     doc->SetAttributeOrRemoveIf(domElement, AttrNotes, notes, notes.isEmpty());
@@ -278,6 +295,8 @@ void VToolCurveIntersectAxis::SaveOptions(QDomElement &tag, QSharedPointer<VGObj
     doc->SetAttribute(tag, AttrAngle, formulaAngle);
     doc->SetAttribute(tag, AttrBasePoint, basePointId);
     doc->SetAttribute(tag, AttrCurve, curveId);
+    doc->SetAttributeOrRemoveIf(tag, AttrAlias1, m_aliasSuffix1, m_aliasSuffix1.isEmpty());
+    doc->SetAttributeOrRemoveIf(tag, AttrAlias2, m_aliasSuffix2, m_aliasSuffix2.isEmpty());
 }
 
 //---------------------------------------------------------------------------------------------------------------------
@@ -285,11 +304,13 @@ void VToolCurveIntersectAxis::ReadToolAttributes(const QDomElement &domElement)
 {
     VToolLinePoint::ReadToolAttributes(domElement);
 
-    m_lineType = doc->GetParametrString(domElement, AttrTypeLine, TypeLineLine);
-    lineColor = doc->GetParametrString(domElement, AttrLineColor, ColorBlack);
-    basePointId = doc->GetParametrUInt(domElement, AttrBasePoint, NULL_ID_STR);
-    curveId = doc->GetParametrUInt(domElement, AttrCurve, NULL_ID_STR);
-    formulaAngle = doc->GetParametrString(domElement, AttrAngle, QString());
+    m_lineType = VAbstractPattern::GetParametrString(domElement, AttrTypeLine, TypeLineLine);
+    lineColor = VAbstractPattern::GetParametrString(domElement, AttrLineColor, ColorBlack);
+    basePointId = VAbstractPattern::GetParametrUInt(domElement, AttrBasePoint, NULL_ID_STR);
+    curveId = VAbstractPattern::GetParametrUInt(domElement, AttrCurve, NULL_ID_STR);
+    formulaAngle = VAbstractPattern::GetParametrString(domElement, AttrAngle, QString());
+    m_aliasSuffix1 = VAbstractPattern::GetParametrEmptyString(domElement, AttrAlias1);
+    m_aliasSuffix2 = VAbstractPattern::GetParametrEmptyString(domElement, AttrAlias2);
 }
 
 //---------------------------------------------------------------------------------------------------------------------
@@ -297,7 +318,7 @@ void VToolCurveIntersectAxis::SetVisualization()
 {
     if (not vis.isNull())
     {
-        VisToolCurveIntersectAxis *visual = qobject_cast<VisToolCurveIntersectAxis *>(vis);
+        auto *visual = qobject_cast<VisToolCurveIntersectAxis *>(vis);
         SCASSERT(visual != nullptr)
 
         visual->setObject1Id(curveId);
@@ -310,161 +331,32 @@ void VToolCurveIntersectAxis::SetVisualization()
 }
 
 //---------------------------------------------------------------------------------------------------------------------
-template <class Item>
-void VToolCurveIntersectAxis::InitArc(VContainer *data, qreal segLength, const VPointF *p, quint32 curveId)
+QString VToolCurveIntersectAxis::MakeToolTip() const
 {
-    QSharedPointer<Item> a1;
-    QSharedPointer<Item> a2;
+    const QSharedPointer<VPointF> first = VAbstractTool::data.GeometricObject<VPointF>(basePointId);
+    const QSharedPointer<VPointF> second = VAbstractTool::data.GeometricObject<VPointF>(m_id);
 
-    const QSharedPointer<Item> arc = data->GeometricObject<Item>(curveId);
-    Item arc1;
-    Item arc2;
+    const QLineF line(static_cast<QPointF>(*first), static_cast<QPointF>(*second));
 
-    if (not VFuzzyComparePossibleNulls(segLength, -1))
-    {
-        arc->CutArc(segLength, arc1, arc2);
-    }
-    else
-    {
-        arc->CutArc(0, arc1, arc2);
-    }
-
-    // Arc highly depend on id. Need for creating the name.
-    arc1.setId(p->id() + 1);
-    arc2.setId(p->id() + 2);
-
-    if (not VFuzzyComparePossibleNulls(segLength, -1))
-    {
-        a1 = QSharedPointer<Item>(new Item(arc1));
-        a2 = QSharedPointer<Item>(new Item(arc2));
-    }
-    else
-    {
-        a1 = QSharedPointer<Item>(new Item());
-        a2 = QSharedPointer<Item>(new Item());
-
-        // Take names for empty arcs from donors.
-        a1->setName(arc1.name());
-        a2->setName(arc2.name());
-    }
-
-    data->AddArc(a1, arc1.id(), p->id());
-    data->AddArc(a2, arc2.id(), p->id());
+    const QString toolTip = QString("<table>"
+                                    "<tr> <td><b>%6:</b> %7</td> </tr>"
+                                    "<tr> <td><b>%1:</b> %2 %3</td> </tr>"
+                                    "<tr> <td><b>%4:</b> %5°</td> </tr>"
+                                    "<tr> <td><b>%8:</b> %9</td> </tr>"
+                                    "<tr> <td><b>%10:</b> %11</td> </tr>"
+                                    "</table>")
+            .arg(tr("Length")) // 1
+            .arg(VAbstractValApplication::VApp()->fromPixel(line.length())) // 2
+            .arg(UnitsToStr(VAbstractValApplication::VApp()->patternUnits(), true), tr("Angle")) // 3, 4
+            .arg(line.angle()) // 5
+            .arg(tr("Label"), second->name(), /* 6, 7 */
+                 tr("Segment 1"), m_segments.first, /* 8, 9 */
+                 tr("Segment 2"), m_segments.second); /* 10, 11 */
+    return toolTip;
 }
 
 //---------------------------------------------------------------------------------------------------------------------
-QT_WARNING_PUSH
-QT_WARNING_DISABLE_GCC("-Wswitch-default")
-void VToolCurveIntersectAxis::InitSegments(GOType curveType, qreal segLength, const VPointF *p, quint32 curveId,
-                                           VContainer *data)
+void VToolCurveIntersectAxis::SetSegments(const QPair<QString, QString> &segments)
 {
-    switch(curveType)
-    {
-        case GOType::EllipticalArc:
-            InitArc<VEllipticalArc>(data, segLength, p, curveId);
-            break;
-        case GOType::Arc:
-            InitArc<VArc>(data, segLength, p, curveId);
-            break;
-        case GOType::CubicBezier:
-        case GOType::Spline:
-        {
-            QSharedPointer<VAbstractBezier> spline1;
-            QSharedPointer<VAbstractBezier> spline2;
-
-            const auto spl = data->GeometricObject<VAbstractCubicBezier>(curveId);
-            QPointF spl1p2, spl1p3, spl2p2, spl2p3;
-            if (not VFuzzyComparePossibleNulls(segLength, -1))
-            {
-                spl->CutSpline(segLength, spl1p2, spl1p3, spl2p2, spl2p3);
-            }
-            else
-            {
-                spl->CutSpline(0, spl1p2, spl1p3, spl2p2, spl2p3);
-            }
-
-            VSpline *spl1 = new VSpline(spl->GetP1(), spl1p2, spl1p3, *p);
-            VSpline *spl2 = new VSpline(*p, spl2p2, spl2p3, spl->GetP4());
-
-            if (not VFuzzyComparePossibleNulls(segLength, -1))
-            {
-                spline1 = QSharedPointer<VAbstractBezier>(spl1);
-                spline2 = QSharedPointer<VAbstractBezier>(spl2);
-            }
-            else
-            {
-                spline1 = QSharedPointer<VAbstractBezier>(new VSpline());
-                spline2 = QSharedPointer<VAbstractBezier>(new VSpline());
-
-                // Take names for empty splines from donors.
-                spline1->setName(spl1->name());
-                spline2->setName(spl2->name());
-
-                delete spl1;
-                delete spl2;
-            }
-
-            data->RegisterUniqueName(spline1);
-            data->AddSpline(spline1, NULL_ID, p->id());
-
-            data->RegisterUniqueName(spline2);
-            data->AddSpline(spline2, NULL_ID, p->id());
-            break;
-        }
-        case GOType::CubicBezierPath:
-        case GOType::SplinePath:
-        {
-            QSharedPointer<VAbstractBezier> splP1;
-            QSharedPointer<VAbstractBezier> splP2;
-
-            const auto splPath = data->GeometricObject<VAbstractCubicBezierPath>(curveId);
-            VSplinePath *splPath1 = nullptr;
-            VSplinePath *splPath2 = nullptr;
-            if (not VFuzzyComparePossibleNulls(segLength, -1))
-            {
-                VPointF *pC = VToolCutSplinePath::CutSplinePath(segLength, splPath, p->name(), &splPath1, &splPath2);
-                delete pC;
-            }
-            else
-            {
-                VPointF *pC = VToolCutSplinePath::CutSplinePath(0, splPath, p->name(), &splPath1, &splPath2);
-                delete pC;
-            }
-
-            SCASSERT(splPath1 != nullptr)
-            SCASSERT(splPath2 != nullptr)
-
-            if (not VFuzzyComparePossibleNulls(segLength, -1))
-            {
-                splP1 = QSharedPointer<VAbstractBezier>(splPath1);
-                splP2 = QSharedPointer<VAbstractBezier>(splPath2);
-            }
-            else
-            {
-                splP1 = QSharedPointer<VAbstractBezier>(new VSplinePath());
-                splP2 = QSharedPointer<VAbstractBezier>(new VSplinePath());
-
-                // Take names for empty spline paths from donors.
-                splP1->setName(splPath1->name());
-                splP2->setName(splPath2->name());
-
-                delete splPath1;
-                delete splPath2;
-            }
-
-            data->RegisterUniqueName(splP1);
-            data->AddSpline(splP1, NULL_ID, p->id());
-
-            data->RegisterUniqueName(splP2);
-            data->AddSpline(splP2, NULL_ID, p->id());
-            break;
-        }
-        case GOType::Point:
-        case GOType::PlaceLabel:
-        case GOType::Unknown:
-            Q_UNREACHABLE();
-            break;
-    }
+    m_segments = segments;
 }
-
-QT_WARNING_POP
