@@ -47,18 +47,16 @@ template <class T> class QSharedPointer;
 
 struct VToolPointOfIntersectionCurvesInitData : VToolSinglePointInitData
 {
-    VToolPointOfIntersectionCurvesInitData()
-        : VToolSinglePointInitData(),
-          firstCurveId(NULL_ID),
-          secondCurveId(NULL_ID),
-          vCrossPoint(VCrossCurvesPoint::HighestPoint),
-          hCrossPoint(HCrossCurvesPoint::LeftmostPoint)
-    {}
-
-    quint32 firstCurveId;
-    quint32 secondCurveId;
-    VCrossCurvesPoint vCrossPoint;
-    HCrossCurvesPoint hCrossPoint;
+    quint32 firstCurveId{NULL_ID};
+    quint32 secondCurveId{NULL_ID};
+    VCrossCurvesPoint vCrossPoint{VCrossCurvesPoint::HighestPoint};
+    HCrossCurvesPoint hCrossPoint{HCrossCurvesPoint::LeftmostPoint};
+    QPair<QString, QString> curve1Segments{};
+    QPair<QString, QString> curve2Segments{};
+    QString curve1AliasSuffix1{};
+    QString curve1AliasSuffix2{};
+    QString curve2AliasSuffix1{};
+    QString curve2AliasSuffix2{};
 };
 
 class VToolPointOfIntersectionCurves : public VToolSinglePoint
@@ -94,6 +92,10 @@ protected:
     virtual void SaveOptions(QDomElement &tag, QSharedPointer<VGObject> &obj) override;
     virtual void ReadToolAttributes(const QDomElement &domElement) override;
     virtual void SetVisualization() override;
+    virtual auto MakeToolTip() const -> QString override;
+
+    void SetCurve1Segments(const QPair<QString, QString> &segments);
+    void SetCurve2Segments(const QPair<QString, QString> &segments);
 private:
     Q_DISABLE_COPY(VToolPointOfIntersectionCurves)
 
@@ -103,6 +105,14 @@ private:
     VCrossCurvesPoint vCrossPoint;
     HCrossCurvesPoint hCrossPoint;
 
+    QPair<QString, QString> m_curve1Segments{};
+    QPair<QString, QString> m_curve2Segments{};
+
+    QString m_curve1AliasSuffix1{};
+    QString m_curve1AliasSuffix2{};
+    QString m_curve2AliasSuffix1{};
+    QString m_curve2AliasSuffix2{};
+
     explicit VToolPointOfIntersectionCurves(const VToolPointOfIntersectionCurvesInitData &initData,
                                             QGraphicsItem *parent = nullptr);
 };
